;
.include "Header.inc"     ;Label und Werte Zuweisungen
.include "Hardware.inc"   ;Hardware Label  Zuweisungen
;
;Programmstart nach RESET  ( Interrupt Vektoren Tabelle )
;
.CSEG                       ;Was ab hier folgt kommt in den FLASH-Speicher
.ORG $0000                  ;Programm beginnt an der FLASH-Adresse 0x0000

 rjmp  _reset               ;RESET-Vector

.include "IRQm8.inc";Restliche Interrupt Vektortabelle

.ORG INT_VECTORS_SIZE       ;Programmadresse nach den ganzen IRQ-Vektoren

;
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;I Erstinitialisierungen
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;
;Stapelzeiger initialisieren (fuer Unterprogramme und / oder Interrupts)
;
_reset:
 ldi  a,High(RAMEND)        ;RAMEND (SRAM) ist in der Definitions-
 out  SPH,a                 ;datei festgelegt  
 ldi  a,Low (RAMEND)
 out  SPL,a
;
;Variablen initialisieren ( r0-r31 haben unbestimmten Inhalt )
;
 clr  flag_reg              ;Allgemeines Flaggenregister
 clr  ovf0_zaehler          ;Timer0 Overflowzaehler
 clr  key_reg               ;Tasterentprellregister
;
;Ports Erstkonfiguration
; 
 ldi  a,$FF                 ;PortB kommplett als...
 out  DDRB,a                ;...Ausgang konfigurieren
 
 sbi  LED_PORT,led.ok       ;OK-LED ausschalten ( Vorerst Pullup )
 sbi  LED_DDR,led.ok        ;PORT-Bit als Ausgang f. OK-LED

 sbi  KEY_PORT,key.bit      ;Pullup f. Taster einschalten
;
; USART Baudrate einstellen
;
 ldi  a,#ubrr               ;f. UBRRH bei 38400 Baud mit 1,8432Hz Quarz
 clr  b                     ;f. UBBRL bei 38400 Baud mit 1,8432Hz Quarz
 out  UBRRH,b               ;Baudrate 38400 mit einem...
 out  UBRRL,a               ;...1,8432Hz Quarz
;
;USART Sendeschieberegisterinterrupt und Sender ( Transmitter ) freigeben
;
 ldi  a,1<<TXCIE|1<<TXEN    ;USART Sendeschieberegisterinterrupt freigeben...
 out  UCSRB,a               ;...und USART-Sendeeinheit einschalten
;
;USART Format ( 8,N,1 ) ist bereits nach dem Reset voreingestellt
;
;
;Timer0 Overflow Interrupt initialisieren
;
 ldi  a,1<<TOIE0            ;Timer0 Overflow Interrupt freigeben...
 out  TIMSK,a               ;...
 ldi  a,1<<CS02             ;Timer0 Vorteiler => 256...
 out  TCCR0,a               ;...
;
;TWI SCL Frequenz einstellen ( ca. 102 kHz mit TWBR=2 ca. 92kHz )
;und TWI-Einheit einschalten, sowie Two-wire Interface Interrupt freigeben
;
 ldi  a,#twbr                ;Ergibt zusammen mit den Vorteilerbits...
 out  TWBR,a                 ;...TWPS=0 im TWSR ca. 102 kHz

 clr  a                      ;Testweise Gernal Call Adress...
 out  TWDR,a                 ;...einstellen...
 out  TWAR,a                 ;...
  
 sei                        ;Globale Interruptfreigabe im CPU-Statusregister
;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;H Hauptprogrammschleife
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;
_main:
;
;Bei Tasterbetaetigung => TWI-START-BEDINGUNG erzeugen und wenn dies durch die
;TWI-Busschnittstelleneinheit erzeugt wurde, den TWI-Status per RS232 senden
;
 sbrs key_reg,key.actuated  ;Taster betaetigt und entprellt?...
 rjmp _idle                 ;...Nein => Weiter im Progrmam
 
;
;TWI START-Bedingung mit Interruptfreigabe. Das quitieren der Tasterbetaetigung
;erfolgt in der TWI-Interrupt 
;
 rcall _twi_start           ;TWI START-Bedingung mit Interruptfreigabe
;
;USART pro Tasterbetaetigung ein Byte senden. Dazu erst die Tasterbetaetigung
;quitieren, wenn das Byte zum Terminalprogramm ( Hterm ) vollstaendig gesendet
;wurde.
;
 
_idle: 
 

 sbr  flag_reg,1<<pla.flg   ;Programmlaufanzeige-Flag quitieren
 rjmp _main                 ;Sprung zum Anfang des Hauptprogramms ( Mainloop )
; 
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;U Beginn der verwendeten Unterprogramme
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;
.include "UP.inc";Unterprogramme
;
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;Q Beginn der verwendeten Interrupt Service Routinen ( ISR )
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;
.include "ISR.inc";Interrupt Service Routinen

.EXIT                       ;Ende des Quelltextes
